﻿using Hotline.Caching.Interfaces;
using Hotline.CallCenter.Calls;
using Hotline.Share.Notifications;
using MediatR;

namespace Hotline.Application.Handlers.CallCenter.FlowControl
{
    public class QueueVisitorToGroupBusyNotificationHandler:INotificationHandler<QueueVisitorToGroupBusyNotification>
    {
        private readonly ICallRepository _callRepository;
        private readonly ICallDetailRepository _callDetailRepository;
        private readonly ICallCacheManager _callCacheManager;

        public QueueVisitorToGroupBusyNotificationHandler(ICallRepository callRepository, ICallDetailRepository callDetailRepository,ICallCacheManager callCacheManager)
        {
            _callRepository = callRepository;
            _callDetailRepository = callDetailRepository;
            _callCacheManager= callCacheManager;
        }

        public async Task Handle(QueueVisitorToGroupBusyNotification notification, CancellationToken cancellationToken)
        {
            var model = await _callRepository.GetAsync(
               x => x.ConversationId == notification.Visitor.Id &&
                    x.FromNo == notification.Visitor.From && x.CreationTime >= DateTime.Now.AddHours(-2),true,x=>x.CreationTime, cancellationToken);
            if (model!=null)
            {
                model.CallStatus = Share.Enums.CallCenter.ECallStatus.InQueue;
                await _callRepository.UpdateAsync(model, cancellationToken);
                var detail = new CallDetail()
                {
                    CallId = model.Id,
                    CallStatus = Share.Enums.CallCenter.ECallStatus.InQueue,
                    OMCallId = notification.Visitor.CallId,
                    ConversationId = notification.Visitor.Id,
                    EventName = notification.Attribute,
                    FromNo = notification.Visitor.From,
                    ToNo = notification.Visitor.To,
                    Remark = notification.Waiting.Group,
                };
                await _callDetailRepository.AddAsync(detail,cancellationToken);
                //处理队列记录
                //var list = _callCacheManager.GetCallQueueList().ToList();
                //list.Add(model);
                //_callCacheManager.AddOrUpdateCallCache(list);
            }
        }
    }
}
