﻿using Hotline.Share.Enums.Settings;
using System.Linq.Expressions;
using Hotline.SeedData;
using SqlSugar;
using XF.Domain.Authentications;
using XF.Domain.Dependency;
using XF.Domain.Entities;

namespace Hotline.Repository.SqlSugar.DataPermissions;

public class DataPermissionFilterBuilder : IDataPermissionFilterBuilder, IScopeDependency
{
    private readonly ISessionContext _sessionContext;

    public DataPermissionFilterBuilder(ISessionContext sessionContext, IDataPermissionManager dataPermissionManager)
    {
        DataPermissionManager = dataPermissionManager;
        _sessionContext = sessionContext;
    }

    public IDataPermissionManager DataPermissionManager { get; }

    public Expression<Func<TEntity, bool>> Build<TEntity>() where TEntity : class, IEntity<string>, IDataPermission, new()
    {
        var userId = _sessionContext.RequiredUserId;
        var orgCode = _sessionContext.RequiredOrgCode;
        var queryFilterType = DataPermissionManager.GetQueryFilter<TEntity>(_sessionContext);
        switch (queryFilterType)
        {
            case ETableAccessLevel.Creator:
                return d => d.CreatorId == userId;
            case ETableAccessLevel.Org:
                return d => d.CreatorOrgCode == orgCode;
            case ETableAccessLevel.OrgAndBelow:
                return d => d.CreatorOrgCode.StartsWith(orgCode);
            case ETableAccessLevel.All:
                return d => true;
            case ETableAccessLevel.Deny:
            default:
                return d => false;
        }
    }

    public Expression<Func<TEntity, bool>> BuildIncludeFlowData<TEntity>() where TEntity : class, IEntity<string>, IDataPermission, IWorkflow, new()
    {
        var userId = _sessionContext.RequiredUserId;
        var orgCode = _sessionContext.RequiredOrgCode;
        var roles = _sessionContext.Roles;
        if (roles != null && roles.Contains(RoleSeedData.AdminRole))
            return d => true;

        return d => SqlFunc.JsonArrayAny(d.AssignUserIds, userId) ||
                    SqlFunc.JsonArrayAny(d.AssignOrgCodes, orgCode);
    }

    //public Expression<Func<TEntity, bool>> BuildIncludeFlowData<TEntity>() where TEntity : class, IEntity<string>, IDataPermission, IWorkflow, new()
    //{
    //    var userId = _sessionContext.RequiredUserId;
    //    var orgCode = _sessionContext.RequiredOrgCode;
    //    var queryFilterType = DataPermissionManager.GetQueryFilter<TEntity>(_sessionContext);

    //    switch (queryFilterType)
    //    {
    //        case EAuthorityType.Create:
    //            return d => d.CreatorId == userId
    //                        || d.AssignUserIds.Contains(userId)
    //                        || d.AssignOrgCodes.Contains(orgCode);
    //        case EAuthorityType.Org:
    //            return d => d.CreatorOrgCode == orgCode
    //                        || d.AssignUserIds.Contains(userId)
    //                        || d.AssignOrgCodes.Contains(orgCode)
    //                ;
    //        case EAuthorityType.OrgAndBelow:
    //            return d => d.CreatorOrgCode.StartsWith(orgCode)
    //                        || d.AssignUserIds.Contains(userId)
    //                        || d.AssignOrgCodes.Contains(orgCode);
    //        case EAuthorityType.All:
    //            return d => true;
    //        default:
    //            return d => d.AssignUserIds.Contains(userId) || d.AssignOrgCodes.Contains(orgCode);
    //    }
    //}

    //private static bool FlowDataFiltering<TEntity>(TEntity entity, string userId, string orgCode, string[] roles) where TEntity : class, IEntity<string>, IDataPermission, IWorkflow, new()
    //{
    //    if (entity.AssignUserIds.Contains(userId)) return true;
    //    foreach (var assignOrgCode in entity.AssignOrgCodes)
    //    {
    //        if (assignOrgCode == orgCode) return true;
    //        var baseOrg = assignOrgCode.Substring(0, 3);
    //        if (orgCode.StartsWith(baseOrg) && orgCode.Length < assignOrgCode.Length)
    //            return true;
    //    }

    //    return false;
    //    //return entity.AssignRoles.Intersect(roles).Any();
    //}
}